@@ -2242,9 +2242,6 @@ access_handler (void *cls,
        * It requires the value of the Sec-WebSocket-Key header of the request.
        * See also: https://tools.ietf.org/html/rfc6455#section-4.2.2
        */
-      MHD_add_response_header (response,
-                               MHD_HTTP_HEADER_CONNECTION,
-                               "Upgrade");
       MHD_add_response_header (response,
                                MHD_HTTP_HEADER_UPGRADE,
                                "websocket");
