@@ -31,7 +31,7 @@
  * @param err the WinSock error code.
  * @return pointer to string description of specified WinSock error.
  */
-const char*
+const char *
 MHD_W32_strerror_winsock_ (int err)
 {
   switch (err)
@@ -271,12 +271,12 @@ MHD_W32_socket_pair_ (SOCKET sockets_pair[2], int non_blk)
     listen_addr.sin_port = 0;   /* same as htons(0) */
     listen_addr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
     if ( ((0 == bind (listen_s,
-                      (struct sockaddr*) &listen_addr,
+                      (struct sockaddr *) &listen_addr,
                       c_addinlen)) &&
           (0 == listen (listen_s,
                         1) ) &&
           (0 == getsockname (listen_s,
-                             (struct sockaddr*) &listen_addr,
+                             (struct sockaddr *) &listen_addr,
                              &addr_len))) )
     {
       SOCKET client_s = socket (AF_INET,
@@ -294,10 +294,10 @@ MHD_W32_socket_pair_ (SOCKET sockets_pair[2], int non_blk)
       }
 
       if ( (0 != ioctlsocket (client_s,
-                              FIONBIO,
+                              (int) FIONBIO,
                               &on_val)) ||
            ( (0 != connect (client_s,
-                            (struct sockaddr*) &listen_addr,
+                            (struct sockaddr *) &listen_addr,
                             c_addinlen)) &&
              (WSAGetLastError () != WSAEWOULDBLOCK)) )
       {
@@ -309,7 +309,7 @@ MHD_W32_socket_pair_ (SOCKET sockets_pair[2], int non_blk)
 
       addr_len = c_addinlen;
       server_s = accept (listen_s,
-                         (struct sockaddr*) &accepted_from_addr,
+                         (struct sockaddr *) &accepted_from_addr,
                          &addr_len);
       if (INVALID_SOCKET == server_s)
       {
@@ -321,19 +321,29 @@ MHD_W32_socket_pair_ (SOCKET sockets_pair[2], int non_blk)
 
       addr_len = c_addinlen;
       if ( (0 == getsockname (client_s,
-                              (struct sockaddr*) &client_addr,
+                              (struct sockaddr *) &client_addr,
                               &addr_len)) &&
-           (accepted_from_addr.sin_family == client_addr.sin_family) &&
            (accepted_from_addr.sin_port == client_addr.sin_port) &&
            (accepted_from_addr.sin_addr.s_addr ==
             client_addr.sin_addr.s_addr) &&
+           (accepted_from_addr.sin_family == client_addr.sin_family) &&
            ( (0 != non_blk) ?
              (0 == ioctlsocket (server_s,
-                                FIONBIO,
+                                (int) FIONBIO,
                                 &on_val)) :
              (0 == ioctlsocket (client_s,
-                                FIONBIO,
-                                &off_val)) ) )
+                                (int) FIONBIO,
+                                &off_val)) ) &&
+           (0 == setsockopt (server_s,
+                             IPPROTO_TCP,
+                             TCP_NODELAY,
+                             (const void *) (&on_val),
+                             sizeof (on_val))) &&
+           (0 == setsockopt (client_s,
+                             IPPROTO_TCP,
+                             TCP_NODELAY,
+                             (const void *) (&on_val),
+                             sizeof (on_val))) )
       {
         closesocket (listen_s);
         sockets_pair[0] = server_s;
@@ -417,7 +427,7 @@ MHD_socket_nonblocking_ (MHD_socket sock)
   unsigned long flags = 1;
 
   if (0 != ioctlsocket (sock,
-                        FIONBIO,
+                        (int) FIONBIO,
                         &flags))
     return 0;
 #endif /* MHD_WINSOCK_SOCKETS */
@@ -516,7 +526,7 @@ MHD_socket_create_listen_ (int pf)
     nosigpipe_set = (SOCK_NOSIGPIPE_OR_ZERO != 0);
 #endif /* SOCK_NOSIGPIPE ||  MHD_socket_nosignal_ */
   }
-#elif defined(MHD_WINSOCK_SOCKETS) && defined (WSA_FLAG_NO_HANDLE_INHERIT)
+#elif defined(MHD_WINSOCK_SOCKETS) && defined(WSA_FLAG_NO_HANDLE_INHERIT)
   fd = WSASocketW (pf,
                    SOCK_STREAM,
                    0,
