@@ -27,6 +27,7 @@
 #include "md5.h"
 #include "test_helpers.h"
 #include <stdio.h>
+#include <stdlib.h>
 
 static int verbose = 0; /* verbose level (0-1)*/
 
@@ -243,16 +244,16 @@ bin2hex (const uint8_t *bin,
 static int
 check_result (const char *test_name,
               unsigned int check_num,
-              const uint8_t calcualted[MD5_DIGEST_SIZE],
+              const uint8_t calculated[MD5_DIGEST_SIZE],
               const uint8_t expected[MD5_DIGEST_SIZE])
 {
-  int failed = memcmp (calcualted, expected, MD5_DIGEST_SIZE);
+  int failed = memcmp (calculated, expected, MD5_DIGEST_SIZE);
   check_num++; /* Print 1-based numbers */
   if (failed)
   {
     char calc_str[MD5_DIGEST_STRING_LENGTH];
     char expc_str[MD5_DIGEST_STRING_LENGTH];
-    bin2hex (calcualted, MD5_DIGEST_SIZE, calc_str);
+    bin2hex (calculated, MD5_DIGEST_SIZE, calc_str);
     bin2hex (expected, MD5_DIGEST_SIZE, expc_str);
     fprintf (stderr,
              "FAILED: %s check %u: calculated digest %s, expected digest %s.\n",
@@ -262,7 +263,7 @@ check_result (const char *test_name,
   else if (verbose)
   {
     char calc_str[MD5_DIGEST_STRING_LENGTH];
-    bin2hex (calcualted, MD5_DIGEST_SIZE, calc_str);
+    bin2hex (calculated, MD5_DIGEST_SIZE, calc_str);
     printf (
       "PASSED: %s check %u: calculated digest %s match expected digest.\n",
       test_name, check_num, calc_str);
@@ -289,10 +290,10 @@ test1_str (void)
     uint8_t digest[MD5_DIGEST_SIZE];
 
     MHD_MD5Init (&ctx);
-    MHD_MD5Update (&ctx, (const uint8_t*) data_units1[i].str_l.str,
+    MHD_MD5Update (&ctx, (const uint8_t *) data_units1[i].str_l.str,
                    data_units1[i].str_l.len);
     MHD_MD5Final (&ctx, digest);
-    num_failed += check_result (__FUNCTION__, i, digest,
+    num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units1[i].digest);
   }
   return num_failed;
@@ -313,7 +314,7 @@ test1_bin (void)
     MHD_MD5Init (&ctx);
     MHD_MD5Update (&ctx, data_units2[i].bin_l.bin, data_units2[i].bin_l.len);
     MHD_MD5Final (&ctx, digest);
-    num_failed += check_result (__FUNCTION__, i, digest,
+    num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units2[i].digest);
   }
   return num_failed;
@@ -334,11 +335,11 @@ test2_str (void)
     size_t part_s = data_units1[i].str_l.len / 4;
 
     MHD_MD5Init (&ctx);
-    MHD_MD5Update (&ctx, (const uint8_t*) data_units1[i].str_l.str, part_s);
-    MHD_MD5Update (&ctx, (const uint8_t*) data_units1[i].str_l.str + part_s,
+    MHD_MD5Update (&ctx, (const uint8_t *) data_units1[i].str_l.str, part_s);
+    MHD_MD5Update (&ctx, (const uint8_t *) data_units1[i].str_l.str + part_s,
                    data_units1[i].str_l.len - part_s);
     MHD_MD5Final (&ctx, digest);
-    num_failed += check_result (__FUNCTION__, i, digest,
+    num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units1[i].digest);
   }
   return num_failed;
@@ -362,13 +363,55 @@ test2_bin (void)
     MHD_MD5Update (&ctx, data_units2[i].bin_l.bin + part_s,
                    data_units2[i].bin_l.len - part_s);
     MHD_MD5Final (&ctx, digest);
-    num_failed += check_result (__FUNCTION__, i, digest,
+    num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units2[i].digest);
   }
   return num_failed;
 }
 
 
+/* Use data set number 7 as it has the longest sequence */
+#define DATA_POS 6
+#define MAX_OFFSET 31
+
+static int
+test_unaligned (void)
+{
+  int num_failed = 0;
+  unsigned int offset;
+  uint8_t *buf;
+  uint8_t *digest_buf;
+
+  const struct data_unit2 *const tdata = data_units2 + DATA_POS;
+
+  buf = malloc (tdata->bin_l.len + MAX_OFFSET);
+  digest_buf = malloc (MD5_DIGEST_SIZE + MAX_OFFSET);
+  if ((NULL == buf) || (NULL == digest_buf))
+    exit (99);
+
+  for (offset = MAX_OFFSET; offset >= 1; --offset)
+  {
+    struct MD5Context ctx;
+    uint8_t *unaligned_digest;
+    uint8_t *unaligned_buf;
+
+    unaligned_buf = buf + offset;
+    memcpy (unaligned_buf, tdata->bin_l.bin, tdata->bin_l.len);
+    unaligned_digest = digest_buf + MAX_OFFSET - offset;
+    memset (unaligned_digest, 0, MD5_DIGEST_SIZE);
+
+    MHD_MD5Init (&ctx);
+    MHD_MD5Update (&ctx, unaligned_buf, tdata->bin_l.len);
+    MHD_MD5Final (&ctx, unaligned_digest);
+    num_failed += check_result (MHD_FUNC_, MAX_OFFSET - offset,
+                                unaligned_digest, tdata->digest);
+  }
+  free (digest_buf);
+  free (buf);
+  return num_failed;
+}
+
+
 int
 main (int argc, char *argv[])
 {
@@ -383,5 +426,7 @@ main (int argc, char *argv[])
   num_failed += test2_str ();
   num_failed += test2_bin ();
 
+  num_failed += test_unaligned ();
+
   return num_failed ? 1 : 0;
 }
