@@ -1,6 +1,7 @@
 /*
  This file is part of libmicrohttpd
  Copyright (C) 2007 Christian Grothoff
+ Copyright (C) 2014-2021 Karlson2k (Evgeny Grin)
 
  libmicrohttpd is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
@@ -23,14 +24,25 @@
  * @brief: daemon TLS alert response test-case
  *
  * @author Sagie Amir
+ * @author Karlson2k (Evgeny Grin)
  */
 
 #include "platform.h"
 #include "microhttpd.h"
 #include "tls_test_common.h"
-#ifdef MHD_HTTPS_REQUIRE_GRYPT
+#ifdef MHD_HTTPS_REQUIRE_GCRYPT
 #include <gcrypt.h>
-#endif /* MHD_HTTPS_REQUIRE_GRYPT */
+#endif /* MHD_HTTPS_REQUIRE_GCRYPT */
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif /* HAVE_SIGNAL_H */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif /* HAVE_TIME_H */
+
 #include "mhd_sockets.h" /* only macros used */
 
 
@@ -41,10 +53,72 @@
 #include <windows.h>
 #endif
 
-extern const char srv_key_pem[];
-extern const char srv_self_signed_cert_pem[];
+static const int TIME_OUT = 2;
+
+static unsigned int num_connects = 0;
+static unsigned int num_disconnects = 0;
+
+
+/**
+ * Pause execution for specified number of milliseconds.
+ * @param ms the number of milliseconds to sleep
+ */
+void
+_MHD_sleep (uint32_t ms)
+{
+#if defined(_WIN32)
+  Sleep (ms);
+#elif defined(HAVE_NANOSLEEP)
+  struct timespec slp = {ms / 1000, (ms % 1000) * 1000000};
+  struct timespec rmn;
+  int num_retries = 0;
+  while (0 != nanosleep (&slp, &rmn))
+  {
+    if (num_retries++ > 8)
+      break;
+    slp = rmn;
+  }
+#elif defined(HAVE_USLEEP)
+  uint64_t us = ms * 1000;
+  do
+  {
+    uint64_t this_sleep;
+    if (999999 < us)
+      this_sleep = 999999;
+    else
+      this_sleep = us;
+    /* Ignore return value as it could be void */
+    usleep (this_sleep);
+    us -= this_sleep;
+  } while (us > 0);
+#else
+  sleep ((ms + 999) / 1000);
+#endif
+}
+
+
+void
+socket_cb (void *cls,
+           struct MHD_Connection *c,
+           void **socket_context,
+           enum MHD_ConnectionNotificationCode toe)
+{
+  struct sckt_notif_cb_param *param = (struct sckt_notif_cb_param *) cls;
+  if (NULL == socket_context)
+    abort ();
+  if (NULL == c)
+    abort ();
+  if (NULL == param)
+    abort ();
+
+  if (MHD_CONNECTION_NOTIFY_STARTED == toe)
+    num_connects++;
+  else if (MHD_CONNECTION_NOTIFY_CLOSED == toe)
+    num_disconnects++;
+  else
+    abort ();
+}
 
-static const int TIME_OUT = 3;
 
 static int
 test_tls_session_time_out (gnutls_session_t session, int port)
@@ -57,7 +131,7 @@ test_tls_session_time_out (gnutls_session_t session, int port)
   if (sd == MHD_INVALID_SOCKET)
   {
     fprintf (stderr, "Failed to create socket: %s\n", strerror (errno));
-    return -1;
+    return 2;
   }
 
   memset (&sa, '\0', sizeof (struct sockaddr_in));
@@ -65,35 +139,40 @@ test_tls_session_time_out (gnutls_session_t session, int port)
   sa.sin_port = htons (port);
   sa.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
 
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) (intptr_t) sd);
-
   ret = connect (sd, (struct sockaddr *) &sa, sizeof (struct sockaddr_in));
 
   if (ret < 0)
   {
     fprintf (stderr, "Error: %s\n", MHD_E_FAILED_TO_CONNECT);
     MHD_socket_close_chk_ (sd);
-    return -1;
+    return 2;
   }
 
+#if (GNUTLS_VERSION_NUMBER + 0 >= 0x030109) && ! defined(_WIN64)
+  gnutls_transport_set_int (session, (int) (sd));
+#else  /* GnuTLS before 3.1.9 or Win64 */
+  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) (intptr_t) (sd));
+#endif /* GnuTLS before 3.1.9 or Win64 */
+
   ret = gnutls_handshake (session);
   if (ret < 0)
   {
     fprintf (stderr, "Handshake failed\n");
     MHD_socket_close_chk_ (sd);
-    return -1;
+    return 2;
   }
 
-  (void) sleep (TIME_OUT + 1);
+  _MHD_sleep (TIME_OUT * 1000 + 1200);
 
   /* check that server has closed the connection */
-  /* TODO better RST trigger */
-  if (send (sd, "", 1, 0) == 0)
+  if (1 == num_disconnects)
   {
     fprintf (stderr, "Connection failed to time-out\n");
     MHD_socket_close_chk_ (sd);
-    return -1;
+    return 1;
   }
+  else if (0 != num_disconnects)
+    abort ();
 
   MHD_socket_close_chk_ (sd);
   return 0;
@@ -106,8 +185,6 @@ main (int argc, char *const *argv)
   int errorCount = 0;
   struct MHD_Daemon *d;
   gnutls_session_t session;
-  gnutls_datum_t key;
-  gnutls_datum_t cert;
   gnutls_certificate_credentials_t xcred;
   int port;
   (void) argc;   /* Unused. Silent compiler warning. */
@@ -117,13 +194,30 @@ main (int argc, char *const *argv)
   else
     port = 3070;
 
-#ifdef MHD_HTTPS_REQUIRE_GRYPT
+#ifdef MHD_SEND_SPIPE_SUPPRESS_NEEDED
+#if defined(HAVE_SIGNAL_H) && defined(SIGPIPE)
+  if (SIG_ERR == signal (SIGPIPE, SIG_IGN))
+  {
+    fprintf (stderr, "Error suppressing SIGPIPE signal.\n");
+    exit (99);
+  }
+#else /* ! HAVE_SIGNAL_H || ! SIGPIPE */
+  fprintf (stderr, "Cannot suppress SIGPIPE signal.\n");
+  /* exit (77); */
+#endif
+#endif /* MHD_SEND_SPIPE_SUPPRESS_NEEDED */
+
+#ifdef MHD_HTTPS_REQUIRE_GCRYPT
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 #ifdef GCRYCTL_INITIALIZATION_FINISHED
   gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 #endif
-#endif /* MHD_HTTPS_REQUIRE_GRYPT */
-  gnutls_global_init ();
+#endif /* MHD_HTTPS_REQUIRE_GCRYPT */
+  if (GNUTLS_E_SUCCESS != gnutls_global_init ())
+  {
+    fprintf (stderr, "Cannot initialize GnuTLS.\n");
+    exit (99);
+  }
   gnutls_global_set_log_level (11);
 
   d = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION
@@ -151,13 +245,13 @@ main (int argc, char *const *argv)
     port = (int) dinfo->port;
   }
 
-  if (0 != setup_session (&session, &key, &cert, &xcred))
+  if (0 != setup_session (&session, &xcred))
   {
     fprintf (stderr, "failed to setup session\n");
     return 1;
   }
   errorCount += test_tls_session_time_out (session, port);
-  teardown_session (session, &key, &cert, xcred);
+  teardown_session (session, xcred);
 
   print_test_result (errorCount, argv[0]);
 
